@@ -11,9 +11,9 @@ int cuErrOpts =CU_VERBOSE;
 int cmor_calendar_c2i(char *calendar, cdCalenType *ical) {
   cmor_add_traceback("cmor_calendar_c2i");
   cmor_is_setup();
-  if (strcmp(calendar,"gregorian")==0)*ical = cdStandard;
-  else if (strcmp(calendar,"standard")==0) *ical = cdStandard;
-  else if (strcmp(calendar,"proleptic_gregorian")==0) *ical = cdMixed;
+  if (strcmp(calendar,"gregorian")==0)*ical = cdMixed;
+  else if (strcmp(calendar,"standard")==0) *ical = cdMixed;
+  else if (strcmp(calendar,"proleptic_gregorian")==0) *ical = cdStandard;
   else if (strcmp(calendar,"noleap")==0) *ical = cdNoLeap;
   else if (strcmp(calendar,"365_day")==0) *ical = cdNoLeap;
   else if (strcmp(calendar,"360_day")==0) *ical = cd360;
@@ -305,18 +305,18 @@ int cmor_check_monotonic(double *values,int length, char *name,int isbounds, int
   int mono;
   int nloop;
   double *values2,tmp;
-  /* for (i=0;i<length;i++) printf("in monotonic: %i, %lf, %i\n",i,values[i],isbounds); */
+  /*for (i=0;i<length;i++) printf("in monotonic: %i, %lf, %i\n",i,values[i],isbounds);*/
   cmor_add_traceback("cmor_check_monotonic");
 
   refaxis = &cmor_tables[cmor_axes[axis_id].ref_table_id].axes[cmor_axes[axis_id].ref_axis_id];
-  if (refaxis->axis=='X') {
+  if ((refaxis->axis=='X')&& (strncmp(refaxis->units,"degrees",7)==0)) {
     treatlon=1;
   }
   /* ok ensure that values are monotonic */
   if (isbounds==1) {
     for (i=0;i<length/2-2;i++) {
       if (((values[2*i]-values[2*i+2])/(values[2*i+2]-values[2*i+4]))<0.) { 
-	if (refaxis->axis=='X') {
+	if ((refaxis->axis=='X') && (strncmp(refaxis->units,"degrees",7)==0)) {
 	  treatlon=1;
 	}
 	else {
