@@ -72,12 +72,12 @@ int cmor_has_required_variable_attributes(int var_id)
 
   
 }
-int cmor_set_variable_attribute(int id, char *attribute_name, char type, void *value)
+int cmor_set_variable_attribute_internal(int id, char *attribute_name, char type, void *value)
 {
   extern cmor_var_t cmor_vars[];
   int i,index;
   char msg[CMOR_MAX_STRING];
-  cmor_add_traceback("cmor_set_variable_attribute");
+  cmor_add_traceback("cmor_set_variable_attribute_internal");
 
   cmor_is_setup();
   index=-1;
@@ -109,6 +109,44 @@ int cmor_set_variable_attribute(int id, char *attribute_name, char type, void *v
   cmor_pop_traceback();
   return 0;
 }
+int cmor_set_variable_attribute(int id, char *attribute_name, char type, void *value)
+{
+  char msg[CMOR_MAX_STRING];
+  cmor_add_traceback("cmor_set_variable_attribute");
+
+  /* First of all we need to see if it is not one of the args you can set by calling cmor_variable */
+  if (
+      (strcmp(attribute_name,"units")==0) ||
+      (strcmp(attribute_name,"missing_values")==0) ||
+      (strcmp(attribute_name,"_FillValue")==0) ||
+      (strcmp(attribute_name,"standard_name")==0) ||
+      (strcmp(attribute_name,"long_name")==0) ||
+      (strcmp(attribute_name,"flag_values")==0) ||
+      (strcmp(attribute_name,"flag_meaning")==0) ||
+      (strcmp(attribute_name,"comment")==0) ||
+      (strcmp(attribute_name,"history")==0) ||
+      (strcmp(attribute_name,"original_name")==0) ||
+      (strcmp(attribute_name,"original_units")==0) ||
+      (strcmp(attribute_name,"positive")==0) ||
+      (strcmp(attribute_name,"cell_methods")==0)
+      )
+    {
+      snprintf(msg,CMOR_MAX_STRING,"variable attribute %s (vor variable %s, table %s) must be set via a call to cmor_variable or it is automaticaly set via the tables",attribute_name,cmor_vars[id].id,cmor_tables[cmor_vars[id].ref_table_id].table_id);
+      cmor_handle_error(msg,CMOR_NORMAL);
+      cmor_pop_traceback();
+      return 1;
+    }
+  /* Before setting the attribute we need to see if the variable has been initialized */
+  if (cmor_vars[id].initialized!=-1) {
+    snprintf(msg,CMOR_MAX_STRING,"attribute %s on variable %s (table %s) will probably not be set as the variable has already been created into the output NetCDF file, please place this call BEFORE any cal to cmor_write",attribute_name,cmor_vars[id].id,cmor_tables[cmor_vars[id].ref_table_id].table_id);
+    cmor_handle_error(msg,CMOR_NORMAL);
+    cmor_pop_traceback();
+    return 1;
+  }
+  cmor_pop_traceback();
+  return cmor_set_variable_attribute_internal(id,attribute_name,type,value);
+}
+
 int cmor_get_variable_attribute(int id, char *attribute_name, void *value)
 {
   extern cmor_var_t cmor_vars[];
@@ -528,7 +566,7 @@ int cmor_update_history(int var_id,char *add){
     tmp[0]='\0';
   }
   snprintf(tmp2,CMOR_MAX_STRING,"%s %s altered by CMOR: %s.",tmp,date,add);
-  cmor_set_variable_attribute(var_id,"history",'c',tmp2);
+  cmor_set_variable_attribute_internal(var_id,"history",'c',tmp2);
   cmor_pop_traceback();
   return 0;
 }
@@ -616,26 +654,28 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
     strncpy(cmor_vars[vrid].id,refvar.out_name,CMOR_MAX_STRING);
   }
 
-  cmor_set_variable_attribute(vrid,"standard_name",'c',refvar.standard_name);
-  cmor_set_variable_attribute(vrid,"long_name",'c',refvar.long_name);
+  cmor_set_variable_attribute_internal(vrid,"standard_name",'c',refvar.standard_name);
+  cmor_set_variable_attribute_internal(vrid,"long_name",'c',refvar.long_name);
   if ((refvar.flag_values!=NULL) && (refvar.flag_values[0]!='\0')) {
-    cmor_set_variable_attribute(vrid,"flag_values",'c',refvar.flag_values);
+    cmor_set_variable_attribute_internal(vrid,"flag_values",'c',refvar.flag_values);
   }
   if ((refvar.flag_meanings!=NULL) && (refvar.flag_meanings[0]!='\0')) {
-    cmor_set_variable_attribute(vrid,"flag_meanings",'c',refvar.flag_meanings);
+    cmor_set_variable_attribute_internal(vrid,"flag_meanings",'c',refvar.flag_meanings);
   }
-/*   cmor_set_variable_attribute(vrid,"realm",'c',refvar.realm); */
-  cmor_set_variable_attribute(vrid,"comment",'c',refvar.comment);
+/*   cmor_set_variable_attribute_internal(vrid,"realm",'c',refvar.realm); */
+  cmor_set_variable_attribute_internal(vrid,"comment",'c',refvar.comment);
   if (strcmp(refvar.units,"?")==0) {
     strncpy(cmor_vars[vrid].ounits,units,CMOR_MAX_STRING);
   }
   else {
     strncpy(cmor_vars[vrid].ounits,refvar.units,CMOR_MAX_STRING);
   }
-  cmor_set_variable_attribute(vrid,"units",'c',cmor_vars[vrid].ounits);
+  if (refvar.type!='c' ) {
+    cmor_set_variable_attribute_internal(vrid,"units",'c',cmor_vars[vrid].ounits);
+  }
   strncpy(cmor_vars[vrid].iunits,units,CMOR_MAX_STRING);
-  if ((original_name!=NULL) && (original_name[0]!='\0')) cmor_set_variable_attribute(vrid,"original_name",'c',original_name);
-  if ((history!=NULL) && (history[0]!='\0')) cmor_set_variable_attribute(vrid,"history",'c',history);
+  if ((original_name!=NULL) && (original_name[0]!='\0')) cmor_set_variable_attribute_internal(vrid,"original_name",'c',original_name);
+  if ((history!=NULL) && (history[0]!='\0')) cmor_set_variable_attribute_internal(vrid,"history",'c',history);
   if ((comment!=NULL) && (comment[0]!='\0')) {
     if (cmor_has_variable_attribute(vrid,"comment")==0) {
       
@@ -648,15 +688,15 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
     else {
       strncpy(msg,comment,CMOR_MAX_STRING);
     }
-    cmor_set_variable_attribute(vrid,"comment",'c',msg);
+    cmor_set_variable_attribute_internal(vrid,"comment",'c',msg);
   }
   if (strcmp(units,refvar.units)!=0) {
-    cmor_set_variable_attribute(vrid,"original_units",'c',units);
+    cmor_set_variable_attribute_internal(vrid,"original_units",'c',units);
     snprintf(msg,CMOR_MAX_STRING,"Converted units from '%s' to '%s'",units,refvar.units);
     cmor_update_history(vrid,msg);
   }
-  cmor_set_variable_attribute(vrid,"cell_methods",'c',refvar.cell_methods);
-  cmor_set_variable_attribute(vrid,"cell_measures",'c',refvar.cell_measures);
+  cmor_set_variable_attribute_internal(vrid,"cell_methods",'c',refvar.cell_methods);
+  cmor_set_variable_attribute_internal(vrid,"cell_measures",'c',refvar.cell_measures);
   /*if ((refvar.positive!='\0') && (positive!=NULL) && (positive[0]!=refvar.positive)) cmor_vars[vrid].sign=-1;*/
   if ((positive!=NULL) && (positive[0]!='\0')) {
     if ((positive[0]!='d') && positive[0]!='u') {
@@ -664,21 +704,21 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
       cmor_handle_error(msg,CMOR_CRITICAL);
     }
     if (refvar.positive=='u') {
-      if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute(vrid,"positive",'c',"up");
+      if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute_internal(vrid,"positive",'c',"up");
       if (positive[0]!='u') {
 	cmor_vars[vrid].sign=-1;
 	cmor_update_history(vrid,"Changed sign");
       }
     }
     else if (refvar.positive=='d') {
-      if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute(vrid,"positive",'c',"down");
+      if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute_internal(vrid,"positive",'c',"down");
       if (positive[0]!='d') {
 	cmor_vars[vrid].sign=-1;
 	cmor_update_history(vrid,"Changed sign");
       }
     }
     else { 
-      snprintf(msg,CMOR_MAX_STRING,"variable '%s' (table %s) you passed positive value:%s, but table does not mention it, will be ignored, if you really want this in your variable output use cmor_set_variable_attribute function",cmor_vars[vrid].id,cmor_tables[cmor_vars[vrid].ref_table_id].table_id,positive); 
+      snprintf(msg,CMOR_MAX_STRING,"variable '%s' (table %s) you passed positive value:%s, but table does not mention it, will be ignored, if you really want this in your variable output use cmor_set_variable_attribute_internal function",cmor_vars[vrid].id,cmor_tables[cmor_vars[vrid].ref_table_id].table_id,positive); 
       cmor_handle_error(msg,CMOR_WARNING);
     }
   }
@@ -689,11 +729,11 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
       }
     if (refvar.positive!='\0') {
       if (refvar.positive=='u') {
-	if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute(vrid,"positive",'c',"up");
+	if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute_internal(vrid,"positive",'c',"up");
 	snprintf(msg,CMOR_MAX_STRING,"you did not provide the 'positive' argument for variable: %s (table %s)",cmor_vars[vrid].id,cmor_tables[cmor_vars[vrid].ref_table_id].table_id);
       }
       else if (refvar.positive=='d') {
-	if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute(vrid,"positive",'c',"down");
+	if (cmor_is_required_variable_attribute(refvar,"positive")==0) cmor_set_variable_attribute_internal(vrid,"positive",'c',"down");
 	snprintf(msg,CMOR_MAX_STRING,"you did not provide the 'positive' argument for variable: %s (table %s)",cmor_vars[vrid].id,cmor_tables[cmor_vars[vrid].ref_table_id].table_id);
       }
       cmor_handle_error(msg,CMOR_CRITICAL);
@@ -891,7 +931,7 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
       else {
 	snprintf(ctmp,CMOR_MAX_STRING,"%s %s",msg,cmor_tables[cmor_axes[laxes_ids[i]].ref_table_id].axes[cmor_axes[laxes_ids[i]].ref_axis_id].out_name);
       }
-      cmor_set_variable_attribute(vrid,"coordinates",'c',&ctmp[0]);
+      cmor_set_variable_attribute_internal(vrid,"coordinates",'c',&ctmp[0]);
     }
     else {
 /*       printf("in else\n"); */
@@ -989,23 +1029,23 @@ int cmor_variable(int *var_id, char *name, char *units, int ndims, int axes_ids[
       cmor_update_history(vrid,msg);
     }
     if (refvar.type=='d') {
-      cmor_set_variable_attribute(vrid,"missing_value",'d',&cmor_vars[vrid].omissing);
-      cmor_set_variable_attribute(vrid,"_FillValue",'d',&cmor_vars[vrid].omissing);
+      cmor_set_variable_attribute_internal(vrid,"missing_value",'d',&cmor_vars[vrid].omissing);
+      cmor_set_variable_attribute_internal(vrid,"_FillValue",'d',&cmor_vars[vrid].omissing);
     }
     else if (refvar.type=='f') {
       afloat = (float)cmor_vars[vrid].omissing;
-      cmor_set_variable_attribute(vrid,"missing_value",'f',&afloat);
-      cmor_set_variable_attribute(vrid,"_FillValue",'f',&afloat);
+      cmor_set_variable_attribute_internal(vrid,"missing_value",'f',&afloat);
+      cmor_set_variable_attribute_internal(vrid,"_FillValue",'f',&afloat);
     }
     else if (refvar.type=='l') {
       along = (long)cmor_vars[vrid].omissing;
-      cmor_set_variable_attribute(vrid,"missing_value",'l',&along);
-      cmor_set_variable_attribute(vrid,"_FillValue",'l',&along);
+      cmor_set_variable_attribute_internal(vrid,"missing_value",'l',&along);
+      cmor_set_variable_attribute_internal(vrid,"_FillValue",'l',&along);
     }
     else if (refvar.type=='i') {
       aint = (int)cmor_vars[vrid].omissing;
-      cmor_set_variable_attribute(vrid,"missing_value",'i',&aint);
-      cmor_set_variable_attribute(vrid,"_FillValue",'i',&aint);
+      cmor_set_variable_attribute_internal(vrid,"missing_value",'i',&aint);
+      cmor_set_variable_attribute_internal(vrid,"_FillValue",'i',&aint);
     }
   }
 
@@ -1275,7 +1315,7 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
   cv_converter *ut_cmor_converter=NULL;
   char local_unit[CMOR_MAX_STRING];
   int n_lower_min=0,n_greater_max=0;
-  double emax,emin;
+  double emax,emin,first_time;
   char msg_min[CMOR_MAX_STRING];
   char msg_max[CMOR_MAX_STRING];
   extern ut_system *ut_read;  
@@ -1608,11 +1648,11 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	ierr = nc_put_vara_double(ncid,avar->time_bnds_nc_id,starts,counts2,tmp_vals);
 	if (ierr != NC_NOERR) {snprintf(msg,CMOR_MAX_STRING,"NetCDF error (%i) writing time bounds for variable '%s', already written in file: %i",ierr,avar->id,avar->ntimes_written);cmor_handle_error(msg,CMOR_CRITICAL);}
 	/* /\* ok first time around the we need to store bounds *\/ */
-	/* if (avar->ntimes_written == 0) { */
-	/*   /\* ok first time we're putting data  in *\/ */
-	/*   avar->first_bound = tmp_vals[0]; */
-	/* } */
-	/* avar->last_bound = tmp_vals[ntimes_passed*2-1]; */
+	if (avar->ntimes_written == 0) {
+	  /* ok first time we're putting data  in */
+	  avar->first_bound = tmp_vals[0];
+	}
+	avar->last_bound = tmp_vals[ntimes_passed*2-1];
 	
 	
 	/* ok since we have bounds we need to set time in the middle */
@@ -1621,10 +1661,11 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	  for (i=0;i<ntimes_passed;i++) {
 	    tmp_vals[i]=(tmp_vals[2*i]+tmp_vals[2*i+1])/2.;
 	  }
+	  first_time = tmp_vals[0]; /*store for later */
 	}
 	else { /* we need to put into tmp_vals the right things */
-	  tmp_vals[ntimes_passed-2]=tmp_vals[0] ; /*store for later */
 	  ierr = cmor_convert_time_values(time_vals,'d',ntimes_passed,&tmp_vals[0],cmor_axes[avar->axes_ids[0]].iunits,msg,msg2,msg2);
+	  first_time = tmp_vals[0] ; /*store for later */
 	}
 	
 	
@@ -1635,12 +1676,12 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	  /* all good in that case */
 	}
 	else {
-	  tmp_vals[0] = tmp_vals[ntimes_passed*2-2];
+	  //tmp_vals[0] = tmp_vals[ntimes_passed*2-2];
 	  tmp_vals[ntimes_passed-1]=tmp_vals[ntimes_passed*2-1];
 	}	/* ok now we need to store first and last stuff */
 	if (avar->ntimes_written == 0) {
 	  /* ok first time we're putting data  in */
-	  avar->first_time = tmp_vals[0];
+	  avar->first_time = first_time;
 	}
 	else {
 	  if (tmp_vals[0]<avar->last_time) {
@@ -1659,8 +1700,8 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	  snprintf(msg,CMOR_MAX_STRING,"time axis must have bounds, please pass them to cmor_write along with time values, variable %s, table %s",avar->id,cmor_tables[avar->ref_table_id].table_id);
 	  cmor_handle_error(msg,CMOR_CRITICAL);
 	}
-	/* avar->first_bound=1.e20; */
-	/* avar->last_bound=1.e20; */
+	avar->first_bound=1.e20;
+	avar->last_bound=1.e20;
 	cmor_get_axis_attribute(avar->axes_ids[0],"units",'c',&msg);
 	cmor_get_cur_dataset_attribute("calendar",msg2);
 	tmp_vals = malloc(ntimes_passed*sizeof(double));
@@ -1693,11 +1734,11 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	starts[1]=0;
 	ierr = nc_put_vara_double(ncid,avar->time_bnds_nc_id,starts,counts2,&cmor_axes[avar->axes_ids[0]].bounds[starts[0]*2]);
 	if (ierr != NC_NOERR) {snprintf(msg,CMOR_MAX_STRING,"NCError (%i: %s) writting time bounds values for variable '%s' (table: %s)",ierr,nc_strerror(ierr),avar->id,cmor_tables[avar->ref_table_id].table_id);cmor_handle_error(msg,CMOR_CRITICAL);}
-	/* /\* ok we need to store first and last bounds *\/ */
-	/* if (avar->ntimes_written==0) { */
-	/*   avar->first_bound=cmor_axes[avar->axes_ids[0]].bounds[starts[0]*2]; */
-	/* } */
-	/* avar->last_bound=cmor_axes[avar->axes_ids[0]].bounds[(starts[0]+counts[0])*2]; */
+	/* ok we need to store first and last bounds */
+	if (avar->ntimes_written==0) {
+	  avar->first_bound=cmor_axes[avar->axes_ids[0]].bounds[starts[0]*2];
+	}
+	avar->last_bound=cmor_axes[avar->axes_ids[0]].bounds[(starts[0]+counts[0])*2-1];
       }
       else {
 	/* checks wether you need bounds or not */
@@ -1705,12 +1746,12 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	  snprintf(msg,CMOR_MAX_STRING,"time axis must have bounds, you defined it w/o any for variable %s (table: %s)",avar->id,cmor_tables[avar->ref_table_id].table_id);
 	  cmor_handle_error(msg,CMOR_CRITICAL);
 	}
-	/* avar->first_bound=1.e20; */
-	/* avar->last_bound=1.e20; */
+	avar->first_bound=1.e20;
+	avar->last_bound=1.e20;
       }
       ierr = nc_put_vara_double(ncid,avar->time_nc_id,starts,counts,&cmor_axes[avar->axes_ids[0]].values[starts[0]]);
       if (ierr != NC_NOERR) {snprintf(msg,CMOR_MAX_STRING,"NCError (%i: %s) writting time values for variable '%s' (table: %s)",ierr,nc_strerror(ierr),avar->id,cmor_tables[avar->ref_table_id].table_id);cmor_handle_error(msg,CMOR_CRITICAL);}
-      /* ok now we need to stroe first and last stuff */
+      /* ok now we need to store first and last stuff */
       if (avar->ntimes_written==0) {
 	avar->first_time = cmor_axes[avar->axes_ids[0]].values[starts[0]];
       }
@@ -1732,8 +1773,8 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	snprintf(msg,CMOR_MAX_STRING,"variable '%s' (table: %s) you are passing %i times but no values and you did not define them via cmor_axis",avar->id,cmor_tables[avar->ref_table_id].table_id,ntimes_passed);
 	cmor_handle_error(msg,CMOR_CRITICAL);
       }
-      /* avar->first_bound=1.e20; */
-      /* avar->last_bound=1.e20; */
+      avar->first_bound=1.e20;
+      avar->last_bound=1.e20;
       if (cmor_axes[avar->axes_ids[ierr]].bounds!=NULL) {
 	/* ok at that stage the recentering must already be done so we just need to write the bounds */
 	counts2[0]=counts[0];
@@ -1742,8 +1783,8 @@ int cmor_write_var_to_file(int ncid,cmor_var_t *avar,void *data,char itype, int
 	starts[1]=0;
 	ierr = nc_put_vara_double(ncid,avar->time_bnds_nc_id,starts,counts2,&cmor_axes[avar->axes_ids[0]].bounds[starts[0]*2]);
 	if (ierr != NC_NOERR) {snprintf(msg,CMOR_MAX_STRING,"NCError (%i: %s) writting time bounds values for variable '%s' (table: %s)",ierr,nc_strerror(ierr),avar->id,cmor_tables[avar->ref_table_id].table_id);cmor_handle_error(msg,CMOR_CRITICAL);}
-	/* avar->first_bound=cmor_axes[avar->axes_ids[0]].bounds[0]; */
-	/* avar->last_bound=cmor_axes[avar->axes_ids[0]].bounds[counts[0]*2]; */
+	avar->first_bound=cmor_axes[avar->axes_ids[0]].bounds[0];
+	avar->last_bound=cmor_axes[avar->axes_ids[0]].bounds[counts[0]*2-1];
       }
       ierr = nc_put_vara_double(ncid,avar->time_nc_id,starts,counts,&cmor_axes[avar->axes_ids[0]].values[starts[0]]);
       if (ierr != NC_NOERR) {snprintf(msg,CMOR_MAX_STRING,"NCError (%i: %s) writting time values for variable '%s' (table: %s)",ierr,nc_strerror(ierr),avar->id,cmor_tables[avar->ref_table_id].table_id);cmor_handle_error(msg,CMOR_CRITICAL);}
